<?php

namespace Drupal\languagewire_translation_provider\Client;

use Drupal\languagewire_translation_provider\Adapter\TMGMT\LanguageMapping;
use Drupal\languagewire_translation_provider\api\Models\Project\ProjectModel;
use Drupal\languagewire_translation_provider\api\Models\Project\ProjectStatusModel;
use Drupal\languagewire_translation_provider\Domain\ProjectTemplate;
use Drupal\languagewire_translation_provider\Platform\Services\ServiceInterface;

/**
 * Api Project Specification with project template.
 */
final class LanguageWireProjectFromTemplateSpecification implements ProjectSpecificationInterface {

  /**
   * Project template.
   *
   * @var \Drupal\languagewire_translation_provider\Domain\ProjectTemplate
   */
  private $projectTemplate;

  /**
   * Source documents.
   *
   * @var int[]
   */
  private $sourceDocuments;

  /**
   * Target language.
   *
   * @var \Drupal\languagewire_translation_provider\Adapter\TMGMT\LanguageMapping
   */
  private $targetLanguage;

  /**
   * Project UUID.
   *
   * @var string
   */
  private $projectUuid;

  /**
   * Title.
   *
   * @var string
   */
  private $title;
  /**
   * Service.
   *
   * @var \Drupal\languagewire_translation_provider\Platform\Services\Service
   */
  private $service;

  /**
   * Deadline.
   *
   * @var \DateTimeInterface
   */
  private $deadline;

  /**
   * Briefing.
   *
   * @var string
   */
  private $briefing;

  /**
   * Constructs a new LanguageWireProjectFromTemplateSpecification object.
   *
   * @param \Drupal\languagewire_translation_provider\Domain\ProjectTemplate $projectTemplate
   *   Project template.
   * @param int[] $sourceDocuments
   *   Source documents.
   * @param \Drupal\languagewire_translation_provider\Adapter\TMGMT\LanguageMapping $targetLanguage
   *   Target language.
   * @param string $projectUuid
   *   Project UUID.
   * @param string $title
   *   Title.
   * @param \Drupal\languagewire_translation_provider\Platform\Services\ServiceInterface $service
   *   Service.
   * @param \DateTimeInterface $deadline
   *   Deadline.
   * @param string $briefing
   *   Briefing.
   */
  public function __construct(
    ProjectTemplate $projectTemplate,
    array $sourceDocuments,
    LanguageMapping $targetLanguage,
    string $projectUuid,
    string $title,
    ServiceInterface $service,
    \DateTimeInterface $deadline,
    string $briefing = NULL
  ) {
    $this->projectTemplate = $projectTemplate;
    $this->sourceDocuments = $sourceDocuments;
    $this->targetLanguage = $targetLanguage;
    $this->projectUuid = $projectUuid;
    $this->title = $title;
    $this->service = $service;
    $this->deadline = $deadline;
    $this->briefing = $briefing;
  }

  /**
   * {@inheritdoc}
   */
  public function toSpecification(ClientInterface $client): ProjectModel {
    $termBaseID = $this->service->termBaseId();

    return ProjectModel::initialize(
      $this->service->id(),
      $this->projectUuid,
      $this->title,
      $this->briefing,
      (string) NULL,
      (string) NULL,
      $this->service->deadline()->toUtc()->format('Y-m-d'),
      $this->service->templateId(),
      $this->getServiceId($client),
      $this->service->translationMemoryId(),
      $termBaseID === 0 ? NULL : $termBaseID,
      $this->service->invoicingAccountId(),
      $this->service->userId(),
      $this->service->workAreaId(),
      ProjectStatusModel::initialize(ProjectStatusModel::DRAFT),
      'https://test'
    );
  }

  /**
   * {@inheritdoc}
   */
  public function projectUuid(): string {
    return $this->projectUuid;
  }

  /**
   * {@inheritdoc}
   */
  public function title(): string {
    return $this->title;
  }

}
